#include <iostream>
#include <string>
#include <chrono>
#include <functional>
#include <chrono>
#include <random>
#include <gazebo/gazebo.hh>
#include <gazebo/physics/physics.hh>
#include <gazebo_ros/node.hpp>
#include <sdf/sdf.hh>
#include <gazebo/common/Plugin.hh>
#include <ignition/math/Vector3.hh>
#include <rclcpp/rclcpp.hpp>
#include <atlantis_msgs/srv/box_pick_up.hpp>
#include <atlantis_msgs/srv/box_pos.hpp>
#include <atlantis_msgs/srv/box_put_down.hpp>
#include <atlantis_msgs/srv/box_update_pos.hpp>
#include <nav_msgs/msg/odometry.hpp>
#include <eigen3/Eigen/Dense>
#include <Eigen/Dense>

using namespace Eigen;
using namespace std;

class TenRandomPos {
public:
    TenRandomPos();
    ~TenRandomPos();
    VectorXd get_random_pos(float);
private:
    VectorXd random_pos[10];
    std::mt19937 gen;
    std::uniform_real_distribution<double> dis;
    int index=0;
    
};

TenRandomPos::TenRandomPos() : gen(std::random_device{}()), dis(0.0, 1.0) {
    for (int i = 0; i < 10; ++i) {
        random_pos[i] = VectorXd(6); // Inizializzazione dei vettori con dimensione 6
    }

    random_pos[0] << 1, 0, 0.5, 0, 0, 0;

    random_pos[1] << -1, 0.5, 0.5, 0, 0, 0;
    random_pos[2] << 3, 4, 0.5, 0, 0, 0;
    random_pos[3] << 7, 2, 0.5, 0, 0, 0;
    random_pos[4] << 8.5, -6, 0.5, 0, 0, 0;
    random_pos[5] << -2, -6, 0.5, 0, 0, 0;
    random_pos[6] << -5, 7, 0.5, 0, 0, 0;
    random_pos[7] << -5, -5, 0.5, 0, 0, 0;
    random_pos[8] << -4, -6, 0.5, 0, 0, 0;
    random_pos[9] << 9, -1, 0.5, 0, 0, 0;
    
}

TenRandomPos::~TenRandomPos() {}

VectorXd TenRandomPos::get_random_pos(float probability) {
    // 20% di probabilità di ritornare una posizione casuale.
        if (dis(gen) < probability) {
            this->index = std::uniform_int_distribution<int>(0, 9)(gen);
            return random_pos[this->index];
        }
    
    // Default: ritorna una posizione casuale a caso.
    // return random_pos[std::uniform_int_distribution<int>(0, 9)(gen)];
    return random_pos[this->index];
}

// int main() {
//     TenRandomPos pos_generator;
//     for (int i = 0; i < 10; ++i) {
//         VectorXd pos = pos_generator.get_random_pos();
//         std::cout << "Posizione " << i + 1 << ": " << pos.transpose() << std::endl;
//     }
//     return 0;
// }


struct RPY_Angle_s {
    double roll, pitch, yaw;

    RPY_Angle_s();
    RPY_Angle_s(double roll, double pitch, double yaw);
    void operator=(const RPY_Angle_s& e_angles);

    ~RPY_Angle_s();
};
RPY_Angle_s::RPY_Angle_s()
:roll(0)
,pitch(0)
,yaw(0)
{}

RPY_Angle_s::RPY_Angle_s(double roll, double pitch, double yaw)
{
  this->roll = roll;
  this->pitch = pitch;
  this->yaw = yaw;
}

RPY_Angle_s::~RPY_Angle_s(){}

void RPY_Angle_s::operator=(const RPY_Angle_s& e_angles)
{
  roll  = e_angles.roll;  
  pitch = e_angles.pitch;  
  yaw   = e_angles.yaw;
}



RPY_Angle_s RPY_To_Quaternion(double x, double y, double z, double w)
{
    RPY_Angle_s angles;

    // roll (x-axis rotation)
    double sinr_cosp = 2 * (w * x + y * z);
    double cosr_cosp = 1 - 2 * (x * x + y * y);
    angles.roll = std::atan2(sinr_cosp, cosr_cosp);

    // pitch (y-axis rotation)
    double sinp = 2 * (w * y - z * x);
    if (std::abs(sinp) >= 1)
        angles.pitch = std::copysign(M_PI / 2, sinp); // use 90 degrees if out of range
    else
        angles.pitch = std::asin(sinp);

    // yaw (z-axis rotation)
    double siny_cosp = 2 * (w * z + x * y);
    double cosy_cosp = 1 - 2 * (y * y + z * z);
    angles.yaw = std::atan2(siny_cosp, cosy_cosp);

    return angles;
}


Matrix4d Vector_To_HomogeneousMatrix(double x, double y, double z, double psi, double theta, double phi)
{
    Matrix4d T;
    
    T(0,0) = cos(phi)*cos(theta);
    T(0,1) = cos(phi)*sin(theta)*sin(psi) - sin(phi)*cos(psi);
    T(0,2) = cos(phi)*sin(theta)*cos(psi) + sin(phi)*sin(psi);
    T(0,3) = x;

    T(1,0) = sin(phi)*cos(theta);
    T(1,1) = sin(phi)*sin(theta)*sin(psi) + cos(phi)*cos(psi);
    T(1,2) = sin(phi)*sin(theta)*cos(psi) - cos(phi)*sin(psi);
    T(1,3) = y;

    T(2,0) = -sin(theta);
    T(2,1) = cos(theta)*sin(psi);
    T(2,2) = cos(theta)*cos(psi);
    T(2,3) = z;

    T(3,0) = 0;
    T(3,1) = 0;
    T(3,2) = 0;
    T(3,3) = 1;
    
    return T;
}

VectorXd HomogeneousMatrix_To_Vector(Matrix4d T){
    Eigen::VectorXd vec(6);
    vec(0) = T(0,3);
    vec(1) = T(1,3);
    vec(2) = T(2,3);
    vec(3) = atan2(T(2,1), T(2,2));
    vec(4) = atan2(-T(2,0), sqrt(pow(T(2,1),2) + pow(T(2,2),2)));
    vec(5) = atan2(T(1,0), T(0,0));
    return vec;
}

namespace gazebo
{
  class ModelPush : public ModelPlugin
  {
  public:
    ModelPush() : ModelPlugin()
    {
      T_robot_box_down = Vector_To_HomogeneousMatrix(0.8, 0, 0.6, 0, 0, 0);
      T_robot_box_up = Vector_To_HomogeneousMatrix(0.0, 0, 0.5, 1.57, 0, 1.57);
      // print in green color PLugin started

      last_update = std::chrono::steady_clock::now();
      this->time = 0;
      this->probability=0;
      this->robot_x_pos = 0;
      this->robot_y_pos = 0;
      this->robot_z_pos = 0;
    }


    ~ModelPush()
    {
      // print in red color PLugin stopped
      RCLCPP_INFO(gazebo_ros_->get_logger(),  "Plugin %s has stopped!", parent_->GetName().c_str());
      
    }


    void Load(physics::ModelPtr _parent, sdf::ElementPtr _sdf)
    {

      parent_ = _parent;
      gazebo_ros_ = gazebo_ros::Node::Get(_sdf);

      // Get QoS profiles
      const gazebo_ros::QoS & qos = gazebo_ros_->get_qos();
      sub_ground_truth_sub_ = gazebo_ros_->create_subscription<nav_msgs::msg::Odometry>( "/ground_truth_odom", 
                                                                                          qos.get_subscription_qos("/ground_truth_odom", rclcpp::QoS(10)), 
                                                                                          std::bind( &ModelPush::callbackGroundTruth, this,std::placeholders::_1 ));
       
      RCLCPP_INFO(gazebo_ros_->get_logger(),  "Plugin %s has started!", parent_->GetName().c_str());
      
      parent_->SetStatic(false);
      // this->box_model->SetGravityMode(false);
      std::string box_name = parent_->GetName();
      for (int i = 0; i < box_name.length(); i++){
        box_name[i] = toupper(box_name[i]);
      }
      std::string service_pickup_name = box_name + "/pick_up";
      std::string service_get_position_name = box_name + "/get_position";
      std::string service_putdown_name = box_name + "/put_down";
      std::string service_update_position_name = box_name + "/update_position";
      
      this->service_pickup_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::BoxPickUp>(
      service_pickup_name,
      std::bind(
        &ModelPush::callbackServicePickUp, this,
        std::placeholders::_1, std::placeholders::_2));

      this->service_get_position_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::BoxPos>(
        service_get_position_name,
        std::bind(
          &ModelPush::callbackServiceGetPosition, this,
          std::placeholders::_1, std::placeholders::_2));
      
      this->service_putdown_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::BoxPutDown>(
      service_putdown_name,
      std::bind(
        &ModelPush::callbackServicePutDown, this,
        std::placeholders::_1, std::placeholders::_2));

    this->service_update_position_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::BoxUpdatePos>(
      service_update_position_name,
      std::bind(
        &ModelPush::callbackServiceUpdatePosition, this,
        std::placeholders::_1, std::placeholders::_2));

      // Listen to the update event. This event is broadcast every simulation iteration.
      this->update_connection_ = event::Events::ConnectWorldUpdateBegin (std::bind (&ModelPush::UpdateChild, this, std::placeholders::_1));
    }

    // Called by the world update start event
    void UpdateChild(const gazebo::common::UpdateInfo & _info)
    {
      if (this->activate){        
        auto now = std::chrono::steady_clock::now();
        auto duration = std::chrono::duration_cast<std::chrono::seconds>(now - last_update).count();
        if (duration >= this->time) {
          last_update = now;
          VectorXd pos = this->pos_generator.get_random_pos(this->probability);
          RCLCPP_INFO(gazebo_ros_->get_logger(),  "Update Box position at (%f, %f, %f, %f, %f, %f)!", pos(0), pos(1), pos(2), pos(3), pos(4), pos(5)); 
          parent_->SetWorldPose(ignition::math::Pose3d(pos(0), pos(1), pos(2), pos(3), pos(4), pos(5)));
        }
      }
    }


    void callbackGroundTruth(const nav_msgs::msg::Odometry::ConstPtr &msg)
    {
      this->robot_x_pos = msg->pose.pose.position.x;
      this->robot_y_pos = msg->pose.pose.position.y;
      this->robot_z_pos = msg->pose.pose.position.z;
      
      RPY_Angle_s angles = RPY_To_Quaternion(msg->pose.pose.orientation.x, msg->pose.pose.orientation.y, msg->pose.pose.orientation.z, msg->pose.pose.orientation.w);
      this->T_robot = Vector_To_HomogeneousMatrix(this->robot_x_pos, this->robot_y_pos, 0, 0, 0, angles.yaw);
      
    }

    bool callbackServiceGetPosition(const std::shared_ptr<atlantis_msgs::srv::BoxPos::Request> request,
          std::shared_ptr<atlantis_msgs::srv::BoxPos::Response> response)
    {
      
      if (!request->request){
        response->x_pos =0;
        response->y_pos =0;
        response->z_pos =0;
        return true;

      }
      
      response->x_pos = parent_->WorldPose().Pos().X();
      response->y_pos = parent_->WorldPose().Pos().Y();
      response->z_pos = parent_->WorldPose().Pos().Z();
      response->yaw = parent_->WorldPose().Rot().Yaw();
      RCLCPP_INFO(gazebo_ros_->get_logger(),  "Box position at (%f, %f, %f, %f)!", response->x_pos, response->y_pos, response->z_pos, response->yaw); 
      
      // res.z_rot = this->box_model->WorldPose().Rot().Euler().Z();

      return true;
      
    }


    bool callbackServicePickUp(const std::shared_ptr<atlantis_msgs::srv::BoxPickUp::Request> request,
          std::shared_ptr<atlantis_msgs::srv::BoxPickUp::Response> response)
    {
      if (!request->action){
        response->success = false;
        return true;
      }

      ignition::math::Pose3d pose = parent_->WorldPose();
      //se la norma di pose lungo x e y è < di 1 allora posso prendere la scatola
      double distance = sqrt(pow(pose.Pos().X() - this->robot_x_pos, 2) + pow(pose.Pos().Y() - this->robot_y_pos, 2));
      if  (distance> 1.5){
        RCLCPP_ERROR(gazebo_ros_->get_logger(),  "Box too far from robot!"); 
        RCLCPP_ERROR(gazebo_ros_->get_logger(),  "Robot position at (%f, %f, %f), Box Position at (%f, %f, %f)", 
        this->robot_x_pos, this->robot_y_pos, this->robot_z_pos,
        pose.Pos().X(),pose.Pos().Y(), pose.Pos().Z()); 
        response->success = false;
        return true;
      }
      this->T_box = T_robot * T_robot_box_up;
      VectorXd vec = HomogeneousMatrix_To_Vector(this->T_box);
      
      parent_->SetWorldPose(ignition::math::Pose3d(vec(0), vec(1), vec(2), vec(3), vec(4), vec(5)));
      
      RCLCPP_INFO(gazebo_ros_->get_logger(),  "Box picked up!"); 
      this->box_is_picked = true;
      response->success = true;
      return true;
    }

    bool callbackServicePutDown(const std::shared_ptr<atlantis_msgs::srv::BoxPutDown::Request> request,
          std::shared_ptr<atlantis_msgs::srv::BoxPutDown::Response> response)
    {
      if(this->box_is_picked == false){

         RCLCPP_ERROR(gazebo_ros_->get_logger(),  "Box not picked up, cannot put down!"); 
        response->success = false;
        return true;
      }
      this->T_box = this->T_robot * this->T_robot_box_down;
      VectorXd vec = HomogeneousMatrix_To_Vector(this->T_box);
      RCLCPP_INFO(gazebo_ros_->get_logger(),  "Update Box position at (%f, %f, %f, %f, %f, %f)!", vec(0), vec(1), vec(2), vec(3), vec(4), vec(5)); 
      parent_->SetWorldPose(ignition::math::Pose3d(vec(0), vec(1), vec(2), vec(3), vec(4), vec(5)));
      response->success = true;
      return true;
    }

    bool callbackServiceUpdatePosition(const std::shared_ptr<atlantis_msgs::srv::BoxUpdatePos::Request> request,
          std::shared_ptr<atlantis_msgs::srv::BoxUpdatePos::Response> response)
    {
      // this->box_model->SetWorldPose(ignition::math::Pose3d(request->x_pos, request->y_pos, request->z_pos, 0, 0, 0));
      if (request->activate)
        RCLCPP_INFO(gazebo_ros_->get_logger(),  "Update Box rdm pos activated!"); 
      else 
        RCLCPP_INFO(gazebo_ros_->get_logger(),  "Update Box rdm pos deactivated!"); 

      if (request->time > 0)
        this->time = request->time;
        
      if (request->probability < 0) {
        this->probability = 0; 
        RCLCPP_WARN(gazebo_ros_->get_logger(),  "Probability must be between 0 and 1, clamped to 0"); 
      }
      else if (request->probability >1) {
        this->probability = 1; 
        RCLCPP_WARN(gazebo_ros_->get_logger(),  "Probability must be between 0 and 1, clamped to 1"); 
        }
      else {
        this->probability = request->probability; 
        RCLCPP_WARN(gazebo_ros_->get_logger(),  "Probability set to %f" , request->probability); 
        }
     

      this->activate = request->activate;
      response->done = true;
      return true;
    }

    // Pointer to the model
    private: 

        gazebo_ros::Node::SharedPtr gazebo_ros_;
        physics::ModelPtr parent_;


        rclcpp::Service<atlantis_msgs::srv::BoxUpdatePos>::SharedPtr service_update_position_srv_;
        rclcpp::Service<atlantis_msgs::srv::BoxPos>::SharedPtr service_get_position_srv_;
        rclcpp::Service<atlantis_msgs::srv::BoxPickUp>::SharedPtr service_pickup_srv_;
        rclcpp::Service<atlantis_msgs::srv::BoxPutDown>::SharedPtr service_putdown_srv_;
        
        rclcpp::Subscription<nav_msgs::msg::Odometry>::SharedPtr sub_ground_truth_sub_;
        bool activate = false;
        bool box_is_picked = false;

        // Pointer to the update event connection
        event::ConnectionPtr update_connection_;
        
        float robot_x_pos, robot_y_pos, robot_z_pos, robot_rot_yaw;
        Eigen::Matrix4d T_robot;
        Eigen::Matrix4d T_box, T_robot_box_down, T_robot_box_up;
        int time; 
        float probability;
        TenRandomPos pos_generator;
        std::chrono::steady_clock::time_point last_update;
  
  
  };

  // Register this plugin with the simulator
  GZ_REGISTER_MODEL_PLUGIN(ModelPush)
}

/*servizio update  e ridare la posizione delle scatole*/