#include <functional>

#include <iostream>
#include <string>
#include <algorithm> 

#include <gazebo/gazebo.hh>
#include <gazebo/physics/physics.hh>
#include <gazebo_ros/node.hpp>
#include <sdf/sdf.hh>
#include <gazebo/common/Plugin.hh>
#include <ignition/math/Vector3.hh>

#include <rclcpp/rclcpp.hpp>
#include <atlantis_msgs/srv/door_command.hpp>
#include <atlantis_msgs/srv/door_status.hpp>

using namespace std;
using std::chrono::seconds;
namespace gazebo
{
  class DoorController : public ModelPlugin{
    public: 
        DoorController() : ModelPlugin()
        {
          
        }

        ~DoorController()
        {
          // print in red
          RCLCPP_INFO(gazebo_ros_->get_logger(),  "Plugin %s has stopped!", parent_->GetName().c_str());
          
        }

        void Load(physics::ModelPtr _parent, sdf::ElementPtr _sdf)
        {
          rclcpp::Rate rate(20);
          // Store the pointer to the model
          parent_ = _parent;
          gazebo_ros_ = gazebo_ros::Node::Get(_sdf);

        
         this->update_connection_ = event::Events::ConnectWorldUpdateBegin (std::bind (&DoorController::UpdateChild, this, std::placeholders::_1));


          this->service_name_ = parent_->GetName() + "/door_command";
          this->service_status_name_ = parent_->GetName() + "/get_status";
          RCLCPP_INFO(gazebo_ros_->get_logger(),  "Plugin %s has started!", parent_->GetName().c_str());
          
            
          this->service_update_door_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::DoorCommand>(
          service_name_,
          std::bind(
            &DoorController::callbackDoorCommand, this,
            std::placeholders::_1, std::placeholders::_2));

          this->service_door_status_srv_ = gazebo_ros_->create_service<atlantis_msgs::srv::DoorStatus>(
            service_status_name_,
            std::bind(
              &DoorController::callbackDoorStatus, this,
              std::placeholders::_1, std::placeholders::_2));

          ignition::math::Pose3d pose = parent_->WorldPose();
          this->initial_pos_x = pose.Pos().X();
          this->initial_pos_y = pose.Pos().Y();
          this->initial_pos_z = pose.Pos().Z();
          this->initial_ang_x = pose.Rot().Euler().X();
          this->initial_ang_y = pose.Rot().Euler().Y();
          this->initial_ang_z = pose.Rot().Euler().Z();
          this->open_quantity = 1.7;

        }

        void UpdateChild(const gazebo::common::UpdateInfo & _info){
            if (parent_->GetName() == "Door3" || parent_->GetName() == "door3"){
              parent_->SetWorldPose(ignition::math::Pose3d(this->initial_pos_x +this->open_quantity  , this->initial_pos_y  , this->initial_pos_z, this->initial_ang_x , this->initial_ang_y , this->initial_ang_z ));
            }
                
            else
                parent_->SetWorldPose(ignition::math::Pose3d(this->initial_pos_x , this->initial_pos_y +this->open_quantity , this->initial_pos_z , this->initial_ang_x , this->initial_ang_y , this->initial_ang_z));
        }

    // Pointer to the model
    private: 
        
        
        gazebo_ros::Node::SharedPtr gazebo_ros_;
        physics::ModelPtr parent_;
        // Pointer to the update event connection
        event::ConnectionPtr update_connection_;
        string service_name_, service_status_name_;
        rclcpp::Service<atlantis_msgs::srv::DoorCommand>::SharedPtr service_update_door_srv_;
        rclcpp::Service<atlantis_msgs::srv::DoorStatus>::SharedPtr service_door_status_srv_;

        float initial_pos_x , initial_pos_y , initial_pos_z , initial_ang_x , initial_ang_y , initial_ang_z = 0;
        bool is_open = false;
        float open_quantity=0;
        

        bool callbackDoorCommand
        (const std::shared_ptr<atlantis_msgs::srv::DoorCommand::Request> request,
          std::shared_ptr<atlantis_msgs::srv::DoorCommand::Response> response)
        {
            RCLCPP_INFO(gazebo_ros_->get_logger(),  "Request to %s %s!", request->command.c_str(),  parent_->GetName().c_str()); 
            std::string command = request->command;
            std::transform(command.begin(), command.end(), command.begin(), ::tolower);
            if (command == "open"){
                this->open_quantity = 1.7;
                if (parent_->GetName() == "Door3" || parent_->GetName() == "door3"){
                  this->open_quantity = 0;
                }
                 this->is_open = true;
                 response->done = true;
                 RCLCPP_INFO(gazebo_ros_->get_logger(),  "Opening %s ...!",  parent_->GetName().c_str()); 
              }
            else if(command == "close"){
                this->open_quantity = 0;
                if (parent_->GetName() == "Door3" || parent_->GetName() == "door3"){
                    this->open_quantity = 2.1;
                }

                this->is_open = false;
                response->done = true;
                RCLCPP_INFO(gazebo_ros_->get_logger(),  "Closing %s...", parent_->GetName().c_str()); 
            }
            else{
              RCLCPP_ERROR(gazebo_ros_->get_logger(),  "Command %s not valid! Valid command are (Open, Close)",request->command); 
              response->done = false;
            }
            
            return true;
        }

        bool callbackDoorStatus
        (const std::shared_ptr<atlantis_msgs::srv::DoorStatus::Request> request,
          std::shared_ptr<atlantis_msgs::srv::DoorStatus::Response> response)
        {
            if(this->is_open){
              RCLCPP_INFO(gazebo_ros_->get_logger(),  "%s is open!", parent_->GetName().c_str()); 
              response->status = "Open";
            }
            else{
              RCLCPP_INFO(gazebo_ros_->get_logger(),  "%s is close!", parent_->GetName().c_str()); 
              response->status = "Close";
            }
            
            return true;
        }
  };

  // Register this plugin with the simulator
  GZ_REGISTER_MODEL_PLUGIN(DoorController)
}
